LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY irs_det IS

	PORT
	(
		b_clock		: IN	STD_LOGIC;
		add_dec		: IN	STD_LOGIC;
		IOR			: IN	STD_LOGIC;
		IR_IN		: IN	STD_LOGIC;
		reset		: IN	STD_LOGIC;
		data		: OUT 	std_logic_vector(11 downto 0);
		flag		: out	std_logic
	);
	
END irs_det;

ARCHITECTURE RTL OF irs_det IS

component shift_reg 
PORT
	(
		sin,enb,reset		: IN	STD_LOGIC;
		clock				: IN 	std_logic;
		p_out				: OUT 	std_logic_vector(23 downto 0)
	);
end component;

component dpll
	PORT
	(
		b_clk		: IN	STD_LOGIC;
		irin		: IN	STD_LOGIC;
		reset		: IN	STD_LOGIC;
		shift_p		: OUT	STD_LOGIC
	);
END component;

	SIGNAL	ir_data_latch	: std_logic_vector(23 downto 0);
	SIGNAL	ir_out			: std_logic_vector(23 downto 0);
	signal  flag_latch,flag_rst		: std_logic;
	signal  s_pls			: std_logic;
	signal  ior_delay		: std_logic_vector(0 to 1);
BEGIN
	DP: dpll PORT MAP (b_clock, IR_IN, reset,s_pls);
	SR: shift_reg PORT MAP (IR_IN,s_pls,reset,b_clock,ir_out);
	PROCESS (b_clock, reset)
	BEGIN

		IF reset = '1' THEN
			flag_latch <= '0';
			ir_data_latch <= "000000000000000000000000";
			ior_delay <= "00";
		ELSIF (b_clock'EVENT AND b_clock = '0') THEN
			ior_delay(0) <= not IOR and add_dec;
			ior_delay(1) <= ior_delay(0);
			if flag_rst = '1' then
				flag_latch <= '0';
			else
				IF ((s_pls = '1') AND
					(ir_out(23) = '1') AND
					(ir_out(21) = '1') AND
					(ir_out(19) = '1') AND
					(ir_out(17) = '1') AND
					(ir_out(15) = '1') AND
					(ir_out(13) = '1') AND
					(ir_out(11) = '1') AND
					(ir_out(9) = '1') AND
					(ir_out(7) = '1') AND
					(ir_out(5) = '1') AND
					(ir_out(3) = '1') AND
					(ir_out(1) = '1') AND
					(flag_latch = '0'))
					THEN				
					ir_data_latch <= ir_out;
					flag_latch <= '1';
				ELSE
					ir_data_latch <= ir_data_latch;
					flag_latch <= flag_latch;
				END IF;
			end if;
		END IF;
		
	END PROCESS;

--	PROCESS (IOR)
--	BEGIN
--	if (IOR = '0' and add_dec ='1') then
--		FOR I IN 0 to 11 LOOP	
--			data(I) <= ir_data_latch(I*2);
--		END LOOP;
--		flag <= flag_latch;
--	else
		--data <= "ZZZZZZZZZZZZ";
		--flag <= 'Z';
--	end if;
--	END PROCESS;
--
	process(ir_data_latch)
	begin
		FOR I IN 0 to 11 LOOP	
			data(I) <= ir_data_latch(I*2);
		END LOOP;
	end process;
	flag <= flag_latch;
--
	flag_rst <= ior_delay(1) and not ior_delay(0);
END RTL;

